==> scmrepo-0.2.0.tar.gz: scmrepo-0.2.0/src/scmrepo/git/backend/dulwich/client.py <==
from typing import Any, Dict, Optional

from dulwich.client import HTTPUnauthorized, Urllib3HttpGitClient

from scmrepo.git.credentials import Credential, CredentialNotFoundError


class GitCredentialsHTTPClient(Urllib3HttpGitClient):  # pylint: disable=abstract-method
    def __init__(
        self,
        base_url,
        username=None,
        password=None,
        config=None,
        **kwargs,
    ):
        super().__init__(
            base_url=base_url,
            username=username,
            password=password,
            config=config,
            **kwargs,
        )
        self._store_credentials: Optional["Credential"] = None

    def _http_request(
        self,
        url: str,
        headers: Optional[Dict[str, str]] = None,
        data: Any = None,
    ):
        try:
            result = super()._http_request(url, headers=headers, data=data)
        except HTTPUnauthorized:
            auth_header = self._get_auth()
            if not auth_header:
                raise
            if headers:
                headers.update(auth_header)
            else:
                headers = auth_header
            result = super()._http_request(url, headers=headers, data=data)
        if self._store_credentials is not None:
            self._store_credentials.approve()
        return result

    def _get_auth(self) -> Dict[str, str]:
        from urllib3.util import make_headers

        try:
            creds = Credential(username=self._username, url=self._base_url).fill()
            self._store_credentials = creds
            return make_headers(basic_auth=f"{creds.username}:{creds.password}")
        except CredentialNotFoundError:
            return {}

==> scmrepo-1.0.0.tar.gz: scmrepo-1.0.0/src/scmrepo/git/backend/dulwich/client.py <==
from typing import Dict, Iterator, List, Optional, Union

from dulwich.client import HTTPUnauthorized, Urllib3HttpGitClient

from scmrepo.git.credentials import Credential, CredentialNotFoundError


class GitCredentialsHTTPClient(Urllib3HttpGitClient):  # pylint: disable=abstract-method
    def __init__(
        self,
        base_url,
        username=None,
        password=None,
        config=None,
        **kwargs,
    ):
        super().__init__(
            base_url=base_url,
            username=username,
            password=password,
            config=config,
            **kwargs,
        )
        self._store_credentials: Optional["Credential"] = None

    def _http_request(
        self,
        url: str,
        headers: Optional[Dict[str, str]] = None,
        data: Optional[Union[bytes, Iterator[bytes]]] = None,
    ):
        cached_chunks: List[bytes] = []

        def _cached_data() -> Iterator[bytes]:
            assert data is not None
            if isinstance(data, bytes):
                yield data
                return

            if cached_chunks:
                yield from cached_chunks
                return

            for chunk in data:
                cached_chunks.append(chunk)
                yield chunk

        try:
            result = super()._http_request(
                url, headers=headers, data=None if data is None else _cached_data()
            )
        except HTTPUnauthorized:
            auth_header = self._get_auth()
            if not auth_header:
                raise
            if headers:
                headers.update(auth_header)
            else:
                headers = auth_header
            result = super()._http_request(
                url, headers=headers, data=None if data is None else _cached_data()
            )
        if self._store_credentials is not None:
            self._store_credentials.approve()
        return result

    def _get_auth(self) -> Dict[str, str]:
        from urllib3.util import make_headers

        try:
            creds = Credential(username=self._username, url=self._base_url).fill()
            self._store_credentials = creds
            return make_headers(basic_auth=f"{creds.username}:{creds.password}")
        except CredentialNotFoundError:
            return {}
